/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity.buildlogic.convention

import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.credentials.HttpHeaderCredentials
import org.gradle.api.publish.PublishingExtension
import org.gradle.api.publish.maven.tasks.AbstractPublishToMaven
import org.gradle.authentication.http.HttpHeaderAuthentication
import org.gradle.kotlin.dsl.create
import org.gradle.kotlin.dsl.credentials
import org.gradle.kotlin.dsl.provideDelegate
import org.gradle.kotlin.dsl.withType
import org.gradle.plugins.signing.Sign
import org.gradle.plugins.signing.SigningExtension
import java.net.URI

class PublicationConventionPlugin : Plugin<Project> {
  override fun apply(project: Project) = with(project) {
    pluginManager.apply("maven-publish")
    extensions.configure(PublishingExtension::class.java) {
      // Configure maven repository
      repositories {
        maven {
          name = "GitlabProjectRepository"
          url = URI("https://git.ucsc.edu/api/v4/projects/12162/packages/maven")
          credentials(HttpHeaderCredentials::class) {
            name = "Job-Token"
            value = System.getenv("CI_JOB_TOKEN")
          }
          authentication {
            create("header", HttpHeaderAuthentication::class)
          }
        }
      }
    }

    pluginManager.apply("signing")
    extensions.configure(SigningExtension::class.java) {
      val signingKeyId: String? by project
      val signingKey: String? by project
      val signingPassword: String? by project
      if (!signingKey.isNullOrEmpty()) {
        useInMemoryPgpKeys(signingKeyId, signingKey, signingPassword)
        extensions.configure(PublishingExtension::class.java) {
          val pubExt = this
          publications {
            sign(pubExt.publications)
          }
        }
      }
    }

    val signingTasks = tasks.withType<Sign>()
    tasks.withType<AbstractPublishToMaven>().configureEach {
      mustRunAfter(signingTasks)
    }
  }
}
