/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
@file:Suppress("unused")

package edu.ucsc.its.temerity.extensions.log

import co.touchlab.kermit.BaseLogger
import co.touchlab.kermit.LoggerConfig
import co.touchlab.kermit.Severity
import org.slf4j.Marker
import org.slf4j.event.Level
import org.slf4j.helpers.AbstractLogger

internal class Slf4jKermitLogger(private val name: String, config: LoggerConfig) : AbstractLogger() {
  private val logger = BaseLogger(config)
  override fun getName(): String = "slf4j-over-kermit"

  //region Is Logging enabled at various levels
  override fun isTraceEnabled() = logger.config.minSeverity <= Severity.Verbose
  override fun isTraceEnabled(marker: Marker?) = logger.config.minSeverity <= Severity.Verbose
  override fun isDebugEnabled() = logger.config.minSeverity <= Severity.Debug
  override fun isDebugEnabled(marker: Marker?) = logger.config.minSeverity <= Severity.Debug
  override fun isInfoEnabled() = logger.config.minSeverity <= Severity.Info
  override fun isInfoEnabled(marker: Marker?) = logger.config.minSeverity <= Severity.Info
  override fun isWarnEnabled() = logger.config.minSeverity <= Severity.Warn
  override fun isWarnEnabled(marker: Marker?) = logger.config.minSeverity <= Severity.Warn
  override fun isErrorEnabled() = logger.config.minSeverity <= Severity.Error
  override fun isErrorEnabled(marker: Marker?) = logger.config.minSeverity <= Severity.Error
  //endregion

  override fun getFullyQualifiedCallerName(): String? = null

  override fun handleNormalizedLoggingCall(
    level: Level?,
    marker: Marker?,
    messagePattern: String?,
    arguments: Array<out Any>?,
    throwable: Throwable?,
  ) {
    val severity = when (level) {
      Level.ERROR -> Severity.Error
      Level.WARN -> Severity.Warn
      Level.INFO -> Severity.Info
      Level.DEBUG -> Severity.Debug
      else -> Severity.Verbose
    }

    val formatted = if (messagePattern != null && arguments != null) {
      String.format(messagePattern, *(arguments.toList().toTypedArray()))
    } else {
      null
    }

    messagePattern.let {
      logger.log(
        severity,
        marker?.toString() ?: name,
        throwable,
        formatted ?: (messagePattern ?: ""),
      )
    }
  }
}
